/* APAuthPolicy.java created by Administrator on Fri 28-Apr-2000 */

import com.apple.yellow.foundation.*;
import com.apple.yellow.eocontrol.*;
import com.apple.yellow.eoaccess.*;
import com.apple.yellow.webobjects.*;
import java.io.*;
import java.util.*;
import java.security.*;
import sun.security.x509.*;
import sun.misc.*;

public class APAuthPolicy extends APAuthPolicyAbstract {

    protected NSMutableDictionary loginFailuresDict;
    static protected APAuthPolicy _apAuthPolicy;

    static public APAuthPolicy authPolicy(){
        if(_apAuthPolicy == null){
            _apAuthPolicy = new APAuthPolicy();
        }
        return _apAuthPolicy;
    }
    

    // Future version using shared applications state will
    // implement the login tracking feature behind these
    // methods
    public APAuthPolicy(){
        super();
        loginFailuresDict = new NSMutableDictionary();
    }
    public int loginFailuresForUserInterval(String username, int interval) {
        return 0;
    }

    // This private call takes the list of resources to test against
    // the resource name to test and a flag to indicate if the
    // list should be treated as the only the resouces to allow
    // or deny
    private boolean shouldDenyAccessToResource(NSArray resources, boolean defaultDenyAll, String resourceName){
        boolean toDeny = true;
        boolean shouldDeny = defaultDenyAll;
        if(defaultDenyAll) toDeny = false;
        int cnt = resources.count();
        int i;
        for(i=0; i<cnt; i++){
            String aName = (String)resources.objectAtIndex(i);
            if(aName.equals(resourceName)){
                shouldDeny = toDeny;
            }
        }
        return shouldDeny;
    }
    
    // If the page name is = to the name for your
    // current login page then this always allows
    // that page. If no page is provided, it uses the
    // defaultDestinationPageName
    public boolean shouldDenyAccessToPageNamed(String pgName){
        boolean deny = true;
        NSArray resources;
        if(pgName.equals(loginPageName())) return false;
        if(pgName == null) pgName = defaultDestinationPageName();
        if(shouldDenyAllPages()){
            resources = pagesToAllow();
            deny = shouldDenyAccessToResource(resources, true, pgName);
        }else{
            resources = pagesToDeny();
            deny = shouldDenyAccessToResource(resources, false, pgName);
        }
        return deny;
    }
   
    // If the action name is = to the name for your
    // current login direct action then this always allows
    // that action. If no page is provided, it uses the
    // defaultDestinationPageName. The loginPanelActionName
    // is also allowed but this is for future functionality
    // and not used in this release
    public boolean shouldDenyAccessToActionNamed (String actionName){
        boolean deny = true;
        NSArray resources;
        if(actionName.equals(loginActionName()) || actionName.equals(APAuthPolicyAbstract.loginPanelActionName) ) return false;
        if(shouldDenyAllActions()){
            resources = actionsToAllow();
            deny = shouldDenyAccessToResource(resources, true, actionName);
        }else{
            resources = actionsToDeny();
            deny = shouldDenyAccessToResource(resources, false, actionName);
        }
        return deny;
    }

    // This method is like a component action or direct action in that it is
    // designed to handle an event. However it is an event that is not specifically
    // asked for by WOF. Another method tests whether this should be called, and
    // if so it provides the role of the loginAction DirectAction.
    // The userInfo passed in here holds additional information about
    // a login attempt failure other than just the boolean true/false
    // For example, some login methods can give reasons for rejecting
    // the user, and this info is placed into this dict. And ultimately this
    // dict is passed to the APLoginPage.
    public boolean handleCertificateRequestForContext (WOContext aContext, NSMutableDictionary userInfo){
        boolean success = false;
        WORequest req = aContext.request();
        // The Base64 encoded cert is retrieved from the request
        String certBlob = req.headerForKey(APAuthPolicyAbstract.CERTIFICATE_HEADER);
        System.out.println("APAuthPolicy: handleCertificateRequestForContext: certBlob:"+ certBlob);
        if(certBlob != null){
            X509Cert cert = null;
            try{
                // If the cert is present, it is decoded and this
                // set of bytes is used to create a new X509Cert object
                sun.misc.BASE64Decoder decoder=new sun.misc.BASE64Decoder();
                cert = new sun.security.x509.X509Cert (decoder.decodeBuffer(new ByteArrayInputStream(certBlob.getBytes()) ));
            }catch(IOException ex) {
                System.out.println("APAuthPolicy: handleCertificateRequestForContext: IOException creating cert");
            }
            System.out.println("APAuthPolicy: handleCertificateRequestForContext: cert:"+ cert);
            // this method will ultimately call your own custom authorization
            // logic in your delegate
            NSObject result = authorizeUserCertificate(cert, certBlob, userInfo);
            if(result != null){
                // If the result is nonNull, we create a session if it aint
                // already built. It then sets the isLoggedIn flag and stuffs
                // the result dict into the session
                System.out.println("APAuthPolicy: handleCertificateRequestForContext: result OK");
                APGuardianSession s = (APGuardianSession) aContext.session();
                s.setIsLoggedIn(true);
                s.setAuthorizationResult(result);
                success = true;
            }
        }
        return success;
    }

    // This method was created initially to prevent a session
    // from being created. However you can acheive this
    // with other settings. It's still here in the event this
    // function or something like it will be needed in the feature
    // In virtually all cases, this method will return false.
    public boolean shouldDenyAccess(WOContext aContext){
        boolean deny = false;
        System.out.println("APAuthPolicy: shouldDenyAccess:");
        if(!aContext.hasSession()){
            WOComponent pg = aContext.page();
            if(pg != null){
                String pgName = pg.name();
                if(!shouldAllowSessionWithoutLogin() && !loginPageName().equals(pgName)){
                    System.out.println("APAuthPolicy: shouldDenyAccess: !shouldAllowSessionWithoutLogin() && !loginPageName().equals(pgName)");
                    deny = true;
                }
            }
        }
        System.out.println("APAuthPolicy: shouldDenyAccess: done: deny:"+ deny);
        return deny;
    }

    // This method provides the basic logic I used to tell
    // if a WORequest came over a SSL link. If the HTTPS header
    // is present, then I know it is. If it's missing, it still might
    // be SSL since not all adaptors/web servers send this. So
    // next I look at the SERVER_PORT header if present. If that is
    // 443 then I guess it is SSL. Note that this is not absolute so
    // you might want to override this method in a subclass for your
    // particular web server and/or SSL port number selection.
    public boolean requestIsSecurePrimitive(WORequest r){
        boolean issecure = false;
        String serverPort = r.headerForKey("SERVER_PORT");
        String https = r.headerForKey("HTTPS");
        if(https != null)
            if(https.equals("ON"))
                issecure = true;
        if(serverPort != null)
            if(serverPort.equals("443"))
                issecure = true;
        return issecure;
    }
    
    public boolean requestIsSecure(WORequest r){
        boolean issecure = false;
        if(delegate != null){
            issecure = delegate.requestIsSecure(r);
        }else{
            issecure = requestIsSecurePrimitive(r);
        }
        return issecure;
    }

    private boolean isInSecureMode(WOContext aContext){
        WORequest r = aContext.request(); 
        return requestIsSecure(r);
    }

    public boolean shouldDenyPageGenerationNotInSecureMode(WOContext aContext){
        WOComponent page = aContext.page();
        String pgName = page.name();
        return shouldDenyPageGenerationNotInSecureMode(aContext, pgName);
    }
    public boolean shouldDenyPageGenerationNotInSecureMode(WOContext aContext, String pgName){
        NSArray resources;
        boolean deny = false;
        boolean isInSecureMode = isInSecureMode(aContext);
        System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: pgName:"+ pgName);
        if(!isInSecureMode){
            System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: is NOT in secure mode");
            if(shouldDenyAllPagesNotInSecureMode()){
                System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: shouldDenyAllPagesNotInSecureMode");
                resources = pagesToAllowNotInSecureMode();
                System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: resources:"+ resources);
                deny = shouldDenyAccessToResource(resources, true, pgName);
            }else{
                System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: !shouldDenyAllPagesNotInSecureMode");
                resources = pagesToDenyNotInSecureMode();
                System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: resources:"+ resources);
                deny = shouldDenyAccessToResource(resources, false, pgName);
            }
        }
        System.out.println("APAuthPolicy: shouldDenyPageGenerationNotInSecureMode: deny:"+ deny);
        return deny;
    }
    public boolean shouldDenyPageGeneration(WOContext aContext){
        boolean deny = shouldDenyAccess(aContext);
        System.out.println("APAuthPolicy: shouldDenyPageGeneration: ");
        if(!deny){
            System.out.println("APAuthPolicy: shouldDenyPageGeneration: should not deny so far...");
            WOComponent page = aContext.page();
            System.out.println("APAuthPolicy: shouldDenyPageGeneration: page:"+ page);
            String pgName = page.name();
            System.out.println("APAuthPolicy: shouldDenyPageGeneration: pgName:"+ pgName);
            if(aContext.hasSession()){
                APGuardianSession s = (APGuardianSession)aContext.session();
                System.out.println("APAuthPolicy: shouldDenyPageGeneration: hasSession: s"+ s);
                if(s.isLoggedIn()) return false;
            }
            deny = shouldDenyAccessToPageNamed(pgName);
        }
        System.out.println("APAuthPolicy: shouldDenyPageGeneration: done: deny:"+ deny);
        return deny;
    }
    public boolean shouldDenyDirectAction(String anActionName, WOContext aContext, APGuardianSession s){
        boolean deny = shouldDenyAccess(aContext);
        if(!deny){
            if(s != null){
                if(s.isLoggedIn()) return false;
            }
            deny = shouldDenyAccessToActionNamed(anActionName);
        }
        return deny;
    }

    
    public APAccessDeniedPage accessDeniedPage(WOContext aContext){
        APAccessDeniedPage nextPage;
        WOApplication app = WOApplication.application();
        nextPage = (APAccessDeniedPage)app.pageWithName(accessDeniedPageName(), aContext);
        return nextPage;
    }

    public APLoginPage loginPage(WOContext aContext){
        APLoginPage nextPage;
        WOApplication app = WOApplication.application();
        nextPage = (APLoginPage)app.pageWithName(loginPageName(), aContext);
        if(!shouldAlwaysUseDefaultDestination()){
            WOComponent dest = aContext.page();
            if(dest != null){
                String destName = dest.name();
                nextPage.setDestinationPageName(destName);
            }
        }
        return nextPage;
    }
    public WOResponse loginResponse(WOResponse aResponse, WOContext aContext){
        WOComponent dest = aContext.page();
        String destName = dest.name();
        return loginResponse(aResponse, aContext, destName);
    }


    private void htmlLoginResponse(WOResponse aResponse, WOContext aContext, String destName){
        APLoginPage lp = loginPage(aContext);
        lp.setDestinationPageName(destName);
        WOResponse rp = lp.generateResponse();
        aResponse.setContent(rp.content());
    }

    private void httpPanelLoginResponseForSessionExpired(WOResponse aResponse, WOContext aContext, String destName){
        APAccessDeniedPage lp = accessDeniedPage (aContext);
        lp.setMessage("Your Session has timed out and you are using HTTP Panel authorization. You must re-enter your application to login");
        lp.setType("SESSION_EXPIRED");
        WOResponse rp = lp.generateResponse();
        aResponse.setContent(rp.content());

        //WORedirect nextPage;
        //WOApplication app = WOApplication.application();
        //nextPage = (WORedirect)app.pageWithName("WORedirect", aContext);
        //nextPage.setURL("http://localhost/WWDC/WebObjects/CFN");
        //WOResponse rp = nextPage.generateResponse();
        //aResponse.setContent(rp.content());
    }
    public void httpPanelLoginResponse(WOResponse aResponse, WOContext aContext, String destName){
        APGuardianSession s = null;
        System.out.println("APAuthPolicy: httpPanelLoginResponse: destName:"+ destName);
        if(aContext.hasSession()){
            s = (APGuardianSession)aContext.session();
            s.setDestinationPageName(destName);
            System.out.println("APAuthPolicy: httpPanelLoginResponse: has session, setting destPageName there");
        }else{
            WOCookie cki = WOCookie.cookieWithName("destinationPageName", destName);
            cki.setPath("/");
            aResponse.addCookie(cki);
            System.out.println("APAuthPolicy: httpPanelLoginResponse: NO session, setting destPageName in Cookie");
        }
        System.out.println("APAuthPolicy: httpPanelLoginResponse: about to call responseForAuthorization");
        responseForAuthorization(aResponse, aContext);
        aResponse.setHeader("no-cache","pragma");
        aResponse.setHeader("Tuesday, 31-Dec-1963 12:00:00 GMT", "expires");
        aResponse.setHeader("-1", "expires");
        System.out.println("APAuthPolicy: httpPanelLoginResponse: done");
    }

    public WOResponse loginResponse(WOResponse aResponse, WOContext aContext, String destName){
        return loginResponse(aResponse, aContext, destName, false);
    }
    public WOResponse loginResponse(WOResponse aResponse, WOContext aContext, String destName, boolean sessionExpired){
        System.out.println("APAuthPolicy: loginResponse: entered");
        if(loginMethod() == APAuthPolicyAbstract.HTML_PAGE){
            System.out.println("APAuthPolicy: loginResponse: HTML_PAGE");
            APLoginPage lp = loginPage(aContext);
            lp.setIsSessionRestorationError(sessionExpired);
            lp.setDestinationPageName(destName);
            WOResponse rp = lp.generateResponse();
            aResponse.setContent(rp.content());
        }else if(loginMethod() == APAuthPolicyAbstract.HTTP_PANEL){
            System.out.println("APAuthPolicy: loginResponse: HTTP_PANEL");
            if(sessionExpired){
                System.out.println("APAuthPolicy: appendToResponse: sessionExpired");
                httpPanelLoginResponseForSessionExpired(aResponse, aContext, destName);
            }else{
                System.out.println("APAuthPolicy: appendToResponse: session NOT Expired");
                httpPanelLoginResponse(aResponse, aContext, destName);
            }
        }else if(loginMethod() == APAuthPolicyAbstract.CERTIFICATE){
            System.out.println("APAuthPolicy: loginResponse: CERTIFICATE");
            System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: CERTIFICATE method");
            System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: not logged in");
            NSMutableDictionary userInfo = new NSMutableDictionary();
            if(handleCertificateRequestForContext(aContext, userInfo)){
                System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: succeeded");
            }else{
                System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: failed");
                WORequest req = aContext.request();
                String certBlob = req.headerForKey(APAuthPolicyAbstract.CERTIFICATE_HEADER);
                if(certBlob != null){
                    System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: should NOT AllowFallbackLoginOnMissingCert");
                    accessDeniedResponse(aResponse, aContext, "Your digital certificate was not granted access", "CERT_LOGIN", userInfo);
                }else{
                    if(!shouldAllowFallbackLoginOnMissingCert()){
                        System.out.println("APAuthPolicy: loginResponse: handleCertificateRequestForContext: should NOT AllowFallbackLoginOnMissingCert");
                        accessDeniedResponse(aResponse, aContext, "This page must be accessed over an SSL connection with a digital certificate", "CERT_LOGIN", userInfo);
                    //return;
                    }else{
                        if(shouldHandleSessionlessAuthorizationRequest(aContext)){
                            System.out.println("APAuthPolicy: loginResponse: will handleAuthorizationRequestForContext");
                            return handleAuthorizationRequestForContext(aContext);
                        }else{
                            System.out.println("APAuthPolicy: loginResponse: shouldAllowFallbackLoginOnMissingCert");
                            if(fallbackLoginMethod() == APAuthPolicyAbstract.HTTP_PANEL){
                                System.out.println("APAuthPolicy: loginResponse: falling back to HTTP_PANEL");
                                httpPanelLoginResponse(aResponse, aContext, destName);
                            }else{
                                System.out.println("APAuthPolicy: loginResponse: falling back to HTML_PANEL");
                                htmlLoginResponse(aResponse, aContext, destName);
                            }
                        }
                    }
                }
            }
        }else{
            System.out.println("APAuthPolicy: loginResponse: else");
            htmlLoginResponse(aResponse, aContext, destName);
        }
        System.out.println("APAuthPolicy: loginResponse: done");
        return aResponse;
    }
    public void accessDeniedResponse(WOResponse aResponse, WOContext aContext, String message, String type, NSMutableDictionary userInfo){
        APAccessDeniedPage lp = accessDeniedPage(aContext);
        lp.setMessage(message);
        lp.setType(type);
        lp.setUserInfo(userInfo);
        WOResponse rp = lp.generateResponse();
        aResponse.setContent(rp.content());
        
    }

    public boolean shouldHandleSessionlessAuthorizationRequest(WOContext aContext){
        return shouldHandleSessionlessAuthorizationRequest(aContext, null);
    }
    public boolean shouldHandleSessionlessAuthorizationRequest(WOContext aContext, WODirectAction da){
        // If there is NO session, BUT there *is* an authorization haeder
        // then we know login isn't successful yet, but has been requested
        // via HTTP challenged on a previous interaction
        boolean hasSession = false;
        boolean isLoggedIn = false;
        if(da != null){
            if(da.existingSession() != null) hasSession = true;
            else hasSession = false;
            isLoggedIn = isLoggedInForDirectAction(da);
        }else{
            hasSession = aContext.hasSession();
            isLoggedIn = isLoggedIn(aContext);
        }
        WORequest aReq = aContext.request();
        String encodedAuth=aReq.headerForKey("authorization");
        System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: encodedAuth:"+ encodedAuth);
        System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: hasSession:"+ hasSession);
        System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: isLoggedIn:"+ isLoggedIn);
        System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: getCookieState(aContext):"+ getCookieState(aContext));
        if(encodedAuth != null && (getCookieState(aContext) || hasSession) && !isLoggedIn){
            if(hasSession){
                APGuardianSession s = (APGuardianSession)aContext.session();
                if(!s.returnedHTTPChallengeResponse()){
                    System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: NO, haven't returned a challenge yet");
                    return false;
                }
                System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: YES, returned a prev challenge");
                return true;
            }
            System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: YES");
            return true;
        }
        System.out.println("APAuthPolicy: shouldHandleSessionlessAuthorizationRequest: NO");
        return false;
    }

    public NSObject handleAuthorizationRequest(WORequest aReq){
        String encodedAuth=null, decodedAuth=null;
        NSObject validatedUser;
        // it could be an EO or NSDictionary

        sun.misc.BASE64Decoder decoder=new sun.misc.BASE64Decoder();
        //encoded string starts after "Basic "
        encodedAuth=aReq.headerForKey("authorization");
        if(encodedAuth==null)
            throw new RuntimeException("No 'authorization' header in request. Is your http server setup to pass on the header?");
        encodedAuth=encodedAuth.substring(encodedAuth.indexOf(" ")+1);
        try{        
            decodedAuth=new String(decoder.decodeBuffer((new ByteArrayInputStream(encodedAuth.getBytes()) )));
        } catch(IOException ex) {System.out.println("IOException while decoding auth header:"+ex);}
        
        //extract username:password, assume methods and ivars exist
        String userName=userNameFromDecodedAuthString(decodedAuth);
        String password=passwordFromDecodedAuthString(decodedAuth);
        System.out.println("APAuthPolicy: handleAuthorizationRequest: userName:>"+ userName +"<");
        System.out.println("APAuthPolicy: handleAuthorizationRequest: password:>"+ password +"<");
        
        //then validate. Again, assume this exists
        System.out.println("APAuthPolicy: handleAuthorizationRequest: about to authorizeUsernamePassword()");
        NSObject result = authorizeUsernamePassword(userName, password);
        return result;
    }
    public WOResponse handleAuthorizationRequestForContext(WOContext c){
        System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: entered");
        String destName = defaultDestinationPageName();
        WORequest aReq = c.request();
        WOResponse aResp = c.response();
        NSObject result = handleAuthorizationRequest(aReq);
        if(c.hasSession()){
            APGuardianSession s = (APGuardianSession)c.session();
            String dn = (String)s.destinationPageName();
            if(dn != null) destName = dn;
        }else{
            String dn = aReq.cookieValueForKey("destinationPageName");
            if(dn != null) destName = dn;
        }
        System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: getCookieState(c):"+ getCookieState(c));
        System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: c.hasSession():"+ c.hasSession());
        System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: result:"+ result);
        if((getCookieState(c) || c.hasSession()) && result != null) {
            System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: SUCCESS && found authCookie or Session");
            APGuardianSession s = (APGuardianSession)c.session(); // creates a new session here if none already created
            s.setIsLoggedIn(true);
            s.setAuthorizationResult(result);
            if(shouldDenyPageGenerationNotInSecureMode(c, destName)){
                accessDeniedResponse(aResp, c, "This page must be accessed over a SSL connection", "SSL_ACCESS", null);
            }else{
                if(loginMethod() != APAuthPolicyAbstract.CERTIFICATE){
                    WOComponent dest = WOApplication.application().pageWithName(destName ,c);
                    aResp.setContent(dest.generateResponse().content());
                }
            }
            System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: about to remove cookie");
            aResp.setHeader("authCookie=0; expires=Sunday, 21-Feb-1966 10:00:00 GMT; path=/;","set-cookie");
        }else{
            if(result != null && !getCookieState(c)){
                System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: SUCCESS && didn't find authCookie");
                accessDeniedResponse(aResp, c, "Cookie support is required to use HTTP Challenge Panels without a session", "COOKIES_REQUIRED", null);
            }else{
                System.out.println("APAuthPolicy: handleAuthorizationRequestForContext: generate a login HTTP panel response");
                responseForAuthorization(aResp, c);
                aResp.setHeader("no-cache","pragma");
                aResp.setHeader("Tuesday, 31-Dec-1963 12:00:00 GMT", "expires");
                aResp.setHeader("-1", "expires");
            }
        }
        return aResp;
    }

    public boolean authCookieExists(WOContext c){
        if(c.request().cookieValueForKey("authCookie")!=null)
            return true;
        else
            return false;
    }
    public void setCookieState(boolean b, WOContext c){
        String s="0";
        if(b)s="1";
        WOCookie authCookie=WOCookie.cookieWithName("authCookie",s);
        authCookie.setPath("/");
        c.response().addCookie(authCookie);
    }
    public boolean getCookieState(WOContext c){
        boolean ret=false;
        if(authCookieExists(c))
            if(c.request().cookieValueForKey("authCookie").equals("1"))
                ret= true;
        return ret;
    }

    // Since this returns null, we default to disallowing access in
    // the absence of an authorization policy (delegate)
    public NSObject authorizeUserCertificate(X509Cert cert, String base64String, NSMutableDictionary userInfo){
        if(delegate != null){
            return delegate.authorizeUserCertificate(cert, base64String, userInfo);
        }
        return null;
    }
    
    public NSObject authorizeUsernamePassword(String uname, String password){
        if(delegate != null){
            return delegate.authorizeUsernamePassword(uname, password);
        }
        return null;
    }
    public WOComponent pageRedirectAfterAuthorization(NSObject result, WOContext context, String destPageName){
        APAuthPolicy ap = APAuthPolicy.authPolicy();
        WOApplication app = WOApplication.application();
        WOComponent pg = app.pageWithName(defaultDestinationPageName(), context);
        if(delegate != null){
            return delegate.pageRedirectAfterAuthorization(result, context, destPageName);
        }
        if(destPageName != null && !shouldAlwaysUseDefaultDestination()){
            pg = app.pageWithName(destPageName, context);
        }
        return pg;
    }

    public static String userNameFromDecodedAuthString(String auth) {
        if(auth!=null){
            StringTokenizer st=new StringTokenizer(auth,":",true);
            String username;
            if(( username=st.nextToken())!=null){
                if(!username.equals(":")){
                    return username;
                }
            }
        }
        return "";
    }
    public static String passwordFromDecodedAuthString(String auth ) {
        if(auth!=null){
            StringTokenizer st=new StringTokenizer(auth,":",true);
            if(st.countTokens()==3){
                st.nextToken(); st.nextToken();
                return st.nextToken();
            }
            else if(st.countTokens()==2){
                st.nextToken();
                String passwd=st.nextToken();
                if(!passwd.equals(":"))
                    return passwd;
            }
        }
        return "";
    }

    public void responseForAuthorization(WOResponse aResponse, WOContext aContext){ 
        System.out.println("APAuthPolicy: responseForAuthorization: entered");
        NSData errorBytes;
        APGuardianSession s = null;
        String errorText = "HTTP/1.0 403 Unauthorized Access";
        String authSpace = new java.util.Date().toString();
        String aRealm = authSpace;
        if(aContext.hasSession()){
            s = (APGuardianSession)aContext.session();
            s.setReturnedHTTPChallengeResponse(true);
        }
        // After you successfully login, this Cookie is removed
        // So when logging in again, it won't be there
        // thus it forces us to login again instead of
        // just riding in for free on the browser's persistent
        // "authorization" header
        System.out.println("APAuthPolicy: responseForAuthorization: about to emit auth Cookie=1");
        setCookieState(true, aContext);
        aResponse.setStatus(401); //authorization request
        aResponse.setHeader("Basic realm=\"" + aRealm + "\"" , "WWW-Authenticate");
        errorBytes = new NSData(errorText.getBytes());
        aResponse.setContent(errorBytes);
        aResponse.setHeader(new Integer(errorBytes.length()).toString(), "content-length");
        aResponse.setHeader("text/html" , "content-type");
        System.out.println("APAuthPolicy: responseForAuthorization: done");
    }

    public boolean isLoggedInForDirectAction(WODirectAction da){
        boolean loggedIn = false;
        APGuardianSession s = (APGuardianSession)da.existingSession();
        if(s != null){
            if(s.isLoggedIn()) loggedIn = true;
        }
        return loggedIn;
    }
    public boolean isLoggedIn(WOContext c){
        boolean loggedIn = false;
        if(c.hasSession()){
            APGuardianSession s = (APGuardianSession)c.session();
            if(s.isLoggedIn()) loggedIn = true;
        }
        return loggedIn;
    }

    public void processResponseForAccess(WOResponse aResponse, WOContext aContext, APResponseGenerator obj){
        System.out.println("APAuthPolicy: processResponseForAccess: entered");
        if(aContext.hasSession()){
            APGuardianSession s = (APGuardianSession) aContext.session();
            WOComponent pg = aContext.page();
            if(pg != null)
                s.setDestinationPageName(pg.name());
        }
        if(loginMethod() == APAuthPolicyAbstract.HTTP_PANEL){
            // this will look for authorization header and log you
            // in if it finds it OK. The response is modified to take
            // you to the login page destination desired
            WORequest aRequest = aContext.request();
            String sid = aRequest.sessionID();
            System.out.println("APAuthPolicy: processResponseForAccess: HTTP_PANEL loginMethod");        
            System.out.println("APAuthPolicy: processResponseForAccess: sid:"+sid);        
            if(shouldHandleSessionlessAuthorizationRequest(aContext)){
                System.out.println("APAuthPolicy: processResponseForAccess: will handleAuthorizationRequestForContext");
                // if not logged in AND there is an Auth header, then we
                // try to login with auth header. If it succeeds we go to default page
                // if it doesn't we reemit the HTTP Panel response
                handleAuthorizationRequestForContext(aContext);
                System.out.println("APAuthPolicy: processResponseForAccess: skipping rest of method");        
                return;
            }
        }
        if(shouldDenyPageGeneration(aContext)){
            System.out.println("APAuthPolicy: processResponseForAccess: shouldDenyPageGeneration");        
            loginResponse(aResponse, aContext);
            if(isLoggedIn(aContext)) obj.finishResponseProcessing (aResponse, aContext);
        }else if(shouldDenyPageGenerationNotInSecureMode(aContext)){
            System.out.println("APAuthPolicy: processResponseForAccess: shouldDenyPageGenerationNotInSecureMode");        
            accessDeniedResponse(aResponse, aContext, "This page must be accessed over a SSL connection", "SSL_ACCESS", null);
        }else{
            System.out.println("APAuthPolicy: processResponseForAccess: about to finish responses");        
            if(obj != null)
                obj.finishResponseProcessing (aResponse, aContext);
        }
        System.out.println("APAuthPolicy: processResponseForAccess: done");        
    }
    

}
