/* APAuthPolicyAbstract.java created by Administrator on Fri 28-Apr-2000 */

import com.apple.yellow.foundation.*;
import com.apple.yellow.eocontrol.*;
import com.apple.yellow.eoaccess.*;
import com.apple.yellow.webobjects.*;

//shouldDisplayAccessWarningPage

public class APAuthPolicyAbstract extends EOCustomObject {

    public static final String CERTIFICATE_HEADER = "CLIENT_CERT";
    public static final String loginPanelActionName = "APDirectAction/loginPanel";

    // Credential presentation method
    public static final int HTML_PAGE = 1;
    public static final int HTTP_PANEL = 2;
    public static final int CERTIFICATE = 3;
    // How will login credentials be presented
    protected int loginMethod;
    // default HTML page for denied access
    protected String accessDeniedPageName = "APAccessDeniedPage";
    // default HTML login page (only applies for HTML)
    protected String loginPageName = "APLoginPage";
    // default directAction for login
    protected String loginActionName = "APDirectAction/login";
    // Login method to use if using Cert, but not connected on HTTPS (only applies for CERTIFICATE)
    protected int fallbackLoginMethod;
    // whether to deny access unless over HTTPS (only applies for CERTIFICATE)
    protected boolean shouldAllowFallbackLoginOnMissingCert = false;
    
    // The default page to goto on successful login
    protected String defaultDestinationPageName = "Main";
    // Force always going to the same page on successful login
    protected boolean shouldAlwaysUseDefaultDestination = false;

    // Session related preferences
    // Goto login page instead of session expired page on session timeout
    protected boolean shouldDisplayLoginOnTimeout = true;
    // Whether to allow a session to be created w/o login
    // Note: we always use a sessionless login page; that feature is not 
    // to be confused with this setting that literally prevents you from 
    // surfing the app unless you have logged in
    protected boolean shouldAllowSessionWithoutLogin = false;

    // Page lock out preferences
    protected NSArray pagesToDeny;
    protected NSArray pagesToAllow;
    protected boolean shouldDenyAllPages = true;
    
    protected NSArray pagesToAllowNotInSecureMode;
    protected NSArray pagesToDenyNotInSecureMode;
    protected boolean shouldDenyAllPagesNotInSecureMode = false;
    
    protected NSArray actionsToDeny;
    protected NSArray actionsToAllow;
    protected boolean shouldDenyAllActions = true;

    // Automatic login functions
    protected boolean shouldAllowAutoLogin = false;
    protected boolean shouldAllowCrossAppLogin = false;

    protected APAuthenticationDelegate delegate = null;

    
    public APAuthPolicyAbstract(){
        super();
        loginMethod = HTML_PAGE; // default to using HTML login page
        pagesToDeny = new NSArray();
        pagesToAllow = new NSArray();
        pagesToAllowNotInSecureMode = new NSArray();
        pagesToDenyNotInSecureMode = new NSArray();
        actionsToDeny = new NSArray();
        actionsToAllow = new NSArray();
        fallbackLoginMethod = HTML_PAGE;
    }

    public NSArray actionsToDeny() {
        return actionsToDeny;
    }
    public void setActionsToDeny(NSArray newActionsToDeny) {
        actionsToDeny = newActionsToDeny;
    }

    public NSArray actionsToAllow() {
        return actionsToAllow;
    }
    public void setActionsToAllow(NSArray newActionsToAllow) {
        actionsToAllow = newActionsToAllow;
    }

    public boolean shouldDenyAllActions() {
        return shouldDenyAllActions;
    }
    public void setShouldDenyAllActions(boolean newShouldDenyAllActions) {
        shouldDenyAllActions = newShouldDenyAllActions;
    }

    public APAuthenticationDelegate delegate () {
        return delegate;
    }
    public void setDelegate(APAuthenticationDelegate v) {
        delegate = v;
    }

    public int loginMethod() {
        return loginMethod;
    }
    public void setLoginMethod(int newLoginMethod) {
        loginMethod = newLoginMethod;
    }

    public boolean shouldDisplayLoginOnTimeout() {
        return shouldDisplayLoginOnTimeout;
    }
    public void setShouldDisplayLoginOnTimeout(boolean newShouldDisplayLoginOnTimeout) {
        shouldDisplayLoginOnTimeout = newShouldDisplayLoginOnTimeout;
    }

    public String loginPageName() {
        return loginPageName;
    }
    public void setLoginPageName(String newLoginPageName) {
        loginPageName = newLoginPageName;
    }

    public NSArray pagesToDeny() {
        return pagesToDeny;
    }
    public void setPagesToDeny(NSArray newPagesToDeny) {
        pagesToDeny = newPagesToDeny;
    }

    public NSArray pagesToAllow() {
        return pagesToAllow;
    }
    public void setPagesToAllow(NSArray newPagesToAllow) {
        pagesToAllow = newPagesToAllow;
    }

    public boolean shouldDenyAllPages() {
        return shouldDenyAllPages;
    }
    public void setShouldDenyAllPages(boolean newShouldDenyAllPages) {
        shouldDenyAllPages = newShouldDenyAllPages;
    }

    public boolean shouldAllowSessionWithoutLogin() {
        return shouldAllowSessionWithoutLogin;
    }
    public void setShouldAllowSessionWithoutLogin(boolean newShouldAllowSessionWithoutLogin) {
        shouldAllowSessionWithoutLogin = newShouldAllowSessionWithoutLogin;
    }

    public String defaultDestinationPageName() {
        return defaultDestinationPageName;
    }
    public void setDefaultDestinationPageName(String newDefaultDestinationPageName) {
        defaultDestinationPageName = newDefaultDestinationPageName;
    }

    public boolean shouldAlwaysUseDefaultDestination() {
        return shouldAlwaysUseDefaultDestination;
    }
    public void setShouldAlwaysUseDefaultDestination(boolean newShouldAlwaysUseDefaultDestination) {
        shouldAlwaysUseDefaultDestination = newShouldAlwaysUseDefaultDestination;
    }

    public boolean shouldAllowAutoLogin() {
        return shouldAllowAutoLogin;
    }
    public void setShouldAllowAutoLogin(boolean newShouldAllowAutoLogin) {
        shouldAllowAutoLogin = newShouldAllowAutoLogin;
    }

    public boolean shouldAllowCrossAppLogin() {
        return shouldAllowCrossAppLogin;
    }
    public void setShouldAllowCrossAppLogin(boolean newShouldAllowCrossAppLogin) {
        shouldAllowCrossAppLogin = newShouldAllowCrossAppLogin;
    }

    public int fallbackLoginMethod() {
        return fallbackLoginMethod;
    }
    public void setFallbackLoginMethod(int newFallbackLoginMethod) {
        fallbackLoginMethod = newFallbackLoginMethod;
    }

    public boolean shouldAllowFallbackLoginOnMissingCert() {
        return shouldAllowFallbackLoginOnMissingCert;
    }
    public void setShouldAllowFallbackLoginOnMissingCert(boolean newShouldAllowFallbackLoginOnMissingCert) {
        shouldAllowFallbackLoginOnMissingCert = newShouldAllowFallbackLoginOnMissingCert;
    }

    public NSArray pagesToAllowNotInSecureMode() {
        return pagesToAllowNotInSecureMode;
    }
    public void setPagesToAllowNotInSecureMode(NSArray newPagesToAllowNotInSecureMode) {
        pagesToAllowNotInSecureMode = newPagesToAllowNotInSecureMode;
    }

    public NSArray pagesToDenyNotInSecureMode() {
        return pagesToDenyNotInSecureMode;
    }
    public void setPagesToDenyNotInSecureMode(NSArray newPagesToDenyNotInSecureMode) {
        pagesToDenyNotInSecureMode = newPagesToDenyNotInSecureMode;
    }

    public boolean shouldDenyAllPagesNotInSecureMode () {
        return shouldDenyAllPagesNotInSecureMode;
    }
    public void setShouldDenyAllPagesNotInSecureMode (boolean newShouldRejectAllPagesNotInSecureMode) {
        shouldDenyAllPagesNotInSecureMode = newShouldRejectAllPagesNotInSecureMode;
    }

    public String loginActionName() {
        return loginActionName;
    }
    public void setLoginActionName(String newLoginActionName) {
        loginActionName = newLoginActionName;
    }
    
    public String accessDeniedPageName() {
        return accessDeniedPageName;
    }
    public void setAccessDeniedPageName(String newAccessDeniedPageName) {
        accessDeniedPageName = newAccessDeniedPageName;
    }

}
